import { not, curry, equals } from 'ramda'
import * as Kefir from 'kefir'
import cursorSelect from './cursorSelect'
import cursorGet from './cursorGet'

/**
 * Set a listener on a data cursor at the given path.
 * When the listener is fired it is given the current value found
 * at that path
 *
 * ** Sig: DataCursor -> EventName -> jsonPath -> Stream **
 *
 * @function
 * @implements curry
 * @memberof lib
 * @param {object} BaobabCursor - The base from which the select path will be made
 * @param {string} eventName - The event that will be listened on
 * @param {string} jsonPath - The JSON path that will traverse the tree
 * @return {object} - Stream - a stream of currentData values from the cursor path
 * @example
 *
 * let cursor = new Baobab({
 *   foo: {
 *     bar: 23
 *   }
 * })
 * let jsonPath = '/foo/bar/'
 *
 * let fooBarStream = lib.cursorOn(cursor, 'update', jsonPath)
 * fooBarStream.onValue(lib.log('/foo/bar is '))
 * lib.cursorSet(cursor, jsonPath, 100) // /foo/bar/ is 100
 *
 */
const cursorOn = curry((cursor, e, jsonPath) => {
  return Kefir.stream(emitter => {

    let past = cursorGet(cursor, jsonPath)
    emitter.emit(past)

    cursorSelect(cursor, jsonPath).on(e, x => {
      let now = x.data.currentData

      if (not(equals(past, now))) {
        past = now
        emitter.emit(now)
      }

    })

  })
})

export default cursorOn
